@@ -28,10 +28,10 @@ template<typename TSeq>
 class Queue;
 
 template<typename TSeq>
-struct Action;
+struct Event;
 
 template<typename TSeq>
-class GlobalAction;
+class GlobalEvent;
 
 template<typename TSeq>
 inline epiworld_double susceptibility_reduction_mixer_default(
@@ -104,7 +104,6 @@ protected:
     bool using_backup = true;
     std::vector< Agent<TSeq> > population_backup = {};
 
-
     /**
      * @name Auxiliary variables for AgentsSample<TSeq> iterators
      * 
@@ -170,8 +169,13 @@ protected:
     epiworld_fast_uint ndays = 0;
     Progress pb;
 
-    std::vector< UpdateFun<TSeq> >    state_fun = {};
-    std::vector< std::string >        states_labels = {};
+    std::vector< UpdateFun<TSeq> >    state_fun = {};                  ///< Functions to update states
+    std::vector< std::string >        states_labels = {};              ///< Labels of the states
+    
+    /** Function to distribute states. Goes along with the function  */
+    std::function<void(Model<TSeq>*)> initial_states_fun = [](Model<TSeq> * /**/)
+    -> void {};
+
     epiworld_fast_uint nstates = 0u;
     
     bool verbose     = true;
@@ -191,20 +195,20 @@ protected:
     void chrono_start();
     void chrono_end();
 
-    std::vector<GlobalAction<TSeq>> global_actions;
+    std::vector<GlobalEvent<TSeq>> globalevents;
 
     Queue<TSeq> queue;
     bool use_queuing   = true;
 
     /**
-     * @brief Variables used to keep track of the actions
+     * @brief Variables used to keep track of the events
      * to be made regarding viruses.
      */
-    std::vector< Action<TSeq> > actions = {};
+    std::vector< Event<TSeq> > events = {};
     epiworld_fast_uint nactions = 0u;
 
     /**
-     * @brief Construct a new Action object
+     * @brief Construct a new Event object
      * 
      * @param agent_ Agent over which the action will be called
      * @param virus_ Virus pointer included in the action
@@ -216,25 +220,18 @@ protected:
      * @param idx_agent_ Location of agent in object.
      * @param idx_object_ Location of object in agent.
      */
-    void actions_add(
+    void events_add(
         Agent<TSeq> * agent_,
         VirusPtr<TSeq> virus_,
         ToolPtr<TSeq> tool_,
         Entity<TSeq> * entity_,
-        epiworld_fast_uint new_state_,
+        epiworld_fast_int new_state_,
         epiworld_fast_int queue_,
         ActionFun<TSeq> call_,
         int idx_agent_,
         int idx_object_
         );
 
-    /**
-     * @brief Executes the stored action
-     * 
-     * @param model_ Model over which it will be executed.
-     */
-    void actions_run();
-
     /**
      * @name Tool Mixers
      * 
@@ -258,12 +255,13 @@ protected:
 
 public:
 
+    
     std::vector<epiworld_double> array_double_tmp;
     std::vector<Virus<TSeq> * > array_virus_tmp;
 
     Model();
     Model(const Model<TSeq> & m);
-    Model(Model<TSeq> & m) = delete;
+    Model(Model<TSeq> & m);
     Model(Model<TSeq> && m);
     Model<TSeq> & operator=(const Model<TSeq> & m);
 
@@ -401,7 +399,7 @@ public:
 
     std::vector< Entity<TSeq> > & get_entities();
 
-    void agents_smallworld(
+    Model<TSeq> & agents_smallworld(
         epiworld_fast_uint n = 1000,
         epiworld_fast_uint k = 5,
         bool d = false,
@@ -423,7 +421,7 @@ public:
     void update_state();
     void mutate_virus();
     void next();
-    virtual void run(
+    virtual Model<TSeq> & run(
         epiworld_fast_uint ndays,
         int seed = -1
     ); ///< Runs the simulation (after initialization)
@@ -438,14 +436,14 @@ public:
         );
     ///@}
 
-    size_t get_n_viruses() const;
-    size_t get_n_tools() const;
+    size_t get_n_viruses() const; ///< Number of viruses in the model
+    size_t get_n_tools() const; ///< Number of tools in the model
     epiworld_fast_uint get_ndays() const;
     epiworld_fast_uint get_n_replicates() const;
     void set_ndays(epiworld_fast_uint ndays);
     bool get_verbose() const;
-    void verbose_off();
-    void verbose_on();
+    Model<TSeq> & verbose_off();
+    Model<TSeq> & verbose_on();
     int today() const; ///< The current time of the model
 
     /**
@@ -525,7 +523,7 @@ public:
      * 
      */
     virtual void reset();
-    void print(bool lite = false) const;
+    const Model<TSeq> & print(bool lite = false) const;
 
     Model<TSeq> && clone() const;
 
@@ -550,6 +548,19 @@ public:
     void print_state_codes() const;
     ///@}
 
+    /**
+     * @name Initial states
+     * 
+     * @details These functions are called before the simulation starts.
+     * 
+     * @param proportions_ Vector of proportions for each state.
+     * @param queue_ Vector of queue for each state.
+     */
+    virtual Model<TSeq> & initial_states(
+        std::vector< double > /*proportions_*/,
+        std::vector< int > /*queue_*/
+    ) {return *this;};
+
     /**
      * @name Setting and accessing parameters from the model
      * 
@@ -626,23 +637,23 @@ public:
      * at the end of every day. Otherwise, the function will be called only
      * at the end of the indicated date.
      */
-    void add_global_action(
+    void add_globalevent(
         std::function<void(Model<TSeq>*)> fun,
         std::string name = "A global action",
         int date = -99
         );
 
-    void add_global_action(
-        GlobalAction<TSeq> action
+    void add_globalevent(
+        GlobalEvent<TSeq> action
     );
 
-    GlobalAction<TSeq> & get_global_action(std::string name); ///< Retrieve a global action by name
-    GlobalAction<TSeq> & get_global_action(size_t i); ///< Retrieve a global action by index
+    GlobalEvent<TSeq> & get_globalevent(std::string name); ///< Retrieve a global action by name
+    GlobalEvent<TSeq> & get_globalevent(size_t i); ///< Retrieve a global action by index
 
-    void rm_global_action(std::string name); ///< Remove a global action by name
-    void rm_global_action(size_t i); ///< Remove a global action by index
+    void rm_globalevent(std::string name); ///< Remove a global action by name
+    void rm_globalevent(size_t i); ///< Remove a global action by index
 
-    void run_global_actions();
+    void run_globalevents();
 
     void clear_state_set();
 
@@ -655,7 +666,7 @@ public:
      */
     ////@{
     void queuing_on(); ///< Activates the queuing system (default.)
-    void queuing_off(); ///< Deactivates the queuing system.
+    Model<TSeq> & queuing_off(); ///< Deactivates the queuing system.
     bool is_queuing_on() const; ///< Query if the queuing system is on.
     Queue<TSeq> & get_queue(); ///< Retrieve the `Queue` object.
     ///@}
@@ -674,6 +685,8 @@ public:
     ///@}
 
     const std::vector< VirusPtr<TSeq> > & get_viruses() const;
+    const std::vector< epiworld_double > & get_prevalence_virus() const;
+    const std::vector< bool > & get_prevalence_virus_as_proportion() const;
     const std::vector< ToolPtr<TSeq> > & get_tools() const;
     Virus<TSeq> & get_virus(size_t id);
     Tool<TSeq> & get_tool(size_t id);
@@ -704,6 +717,14 @@ public:
     bool operator==(const Model<TSeq> & other) const;
     bool operator!=(const Model<TSeq> & other) const {return !operator==(other);};
 
+    /**
+     * @brief Executes the stored action
+     * 
+     * @param model_ Model over which it will be executed.
+     */
+    void events_run();
+
+
 };
 
 #endif
